%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%
-module(erl.lang.binary).

-export([from_list/1,
	 from_term/1,
	 from_term/2,
	 split/2,
	 to_list/1,
	 to_list/3,
	 to_term/1]).


%% @spec (binary()) -> [integer()]
%% @doc Returns a list of integers which correspond to the bytes of
%% the binary.
%% @see to_list/2

to_list(Bin) ->
    erl.lang.binary:to_list(Bin).	% Built-in function


%% @spec (binary(), Start::integer(), End::integer()) -> [integer()]
%% @doc Returns a list of integers which correspond to the specified
%% bytes of the binary. As <code>to_list/1</code>, but returns
%% only the bytes from position <code>Start</code> to position
%% <code>End</code>. Positions are numbered from 1.
%% @see to_list/1

to_list(Bin, Start, Stop) ->
    erl.lang.binary:to_list(Bin, Start, Stop).	% Built-in function

%% @spec (List) -> binary()
%% @doc Concatenates the integers and binaries in the list. The list may
%% be deep and can contain any combination of integers and binaries.
%%
%% <p>Example:
%% <pre>list_to_binary([Bin1,1,[2,3,Bin2],4|Bin3])</pre>
%% </p>
%%
%% <p>Fails with reason <code>badarg</code> if the argument is not a
%% list, or if the list or any sublist contains anything else than
%% binaries or integers in the range [0, 255].</p>

from_list(List) ->
    erl.lang.binary:from_list(List).	% Built-in function


%% @spec (binary()) -> term()
%% @doc Decodes the contents of the binary from the external Erlang term
%% format.
%% @see from_term/1

to_term(Bin) ->
    erl.lang.binary:to_term(Bin).	% Built-in function


%% @spec (term()) -> binary()
%% @doc Encodes an Erlang term in the external Erlang term format.
%% @see from_term/2
%% @see to_term/1

from_term(Term) ->
    erl.lang.binary:from_term(Term).	% Built-in function


%% @spec (Term::term(), Options::[term()]) -> binary()

%% @doc Encodes an Erlang term in the external Erlang term format. If
%% the <code>Options</code> list contains the atom
%% <code>compressed</code>, the external term format will be compressed.
%% The compressed format is automatically recognized by
%% <code>to_term/1</code>.
%% @see to_term/1

from_term(Term, Options) ->
    erl.lang.binary:from_term(Term, Options).	% Built-in function


%% @spec (binary(), integer()) -> {binary(), binary()}
%% @doc Split a binary at a given byte position. Positions are numbered
%% from 1.

split(Bin, Pos) ->
    erl.lang.binary:split(Bin, Pos).	% Built-in function
