%%% -*- Erlang -*-
%%%-------------------------------------------------------------------
%%% File    : hipe_ssa_dead_code.inc
%%% Author  : Per Gustafsson <pergu@it.uu.se>
%%% Description : 
%%%
%%% Created : 11 Aug 2003 by Per Gustafsson <pergu@it.uu.se>
%%%-------------------------------------------------------------------

-export([remove_dead_code/1]).

remove_dead_code(CFG) ->
  Lbls = ?cfg:reverse_postorder(CFG),  
  Liveness=?liveness:analyze(CFG),
  case do_lbls(Lbls, CFG, Liveness, false) of
    {CFG1, true} ->
      remove_dead_code(CFG1);
    {CFG1, false} ->
      CFG1
  end.

do_lbls([Lbl|Rest], CFG, Liveness, Changed) ->
  LiveOut = gb_sets:from_list(?liveness:liveout(Liveness, Lbl)),
  BB = ?cfg:bb(CFG, Lbl),
  Code = hipe_bb:code(BB),
  case do_code(lists:reverse(Code), LiveOut, Changed, []) of
    {NewCode, NewChanged} ->   
      NewBB = hipe_bb:code_update(BB, NewCode),
      NewCFG = ?cfg:bb_update(CFG, Lbl, NewBB),
      do_lbls(Rest, NewCFG, Liveness, NewChanged)
  end;
do_lbls([], CFG, _Liveness, Changed) ->
  {CFG, Changed}.

do_code([Instr|Instrs], LiveOut, Changed, Acc) ->
  Def = ?code:defines(Instr),
  Use = ?code:uses(Instr),
  DefSet = gb_sets:from_list(Def),
  UseSet = gb_sets:from_list(Use),
  LiveIn = gb_sets:union(gb_sets:difference(LiveOut, DefSet), UseSet),
  case gb_sets:is_subset(DefSet, LiveOut) of
    true ->
      do_code(Instrs, LiveIn, Changed, [Instr|Acc]);
    false ->
      case ?code:is_pure(Instr) of
	true ->
	  case ?code:is_call(Instr) of
	    true ->
	      case ?code:call_continuation(Instr) of
		[] ->
		  do_code(Instrs, LiveOut, true, Acc);
		SuccLblName ->
		  NewInstr = ?code:mk_goto(SuccLblName),
		  do_code(Instrs, LiveOut, true, [NewInstr|Acc])
	      end;
	    false ->
	      do_code(Instrs, LiveOut, true, Acc)
	  end;
	false ->
	  case ?code:is_call(Instr) of
	    true ->
	      case ?code:call_dst(Instr) of
		[_Dst] -> %% Remove the dead destination
		  NewInstr = ?code:call_dst_update(Instr, []),
		  do_code(Instrs, LiveIn, true, [NewInstr|Acc]);
		_ ->
		  do_code(Instrs, LiveIn, Changed, [Instr|Acc])
	      end;
	    false ->
	      do_code(Instrs, LiveIn, Changed, [Instr|Acc])
	  end
      end
  end;
do_code([], _LiveOut, Changed, Acc) ->
  {Acc, Changed}.

